;====================================
; PROJECT : 6 CHANNEL RX IR REMOTE
;====================================
#include p18f452.inc

#DEFINE D1	PORTB,0		; display 1
#DEFINE D2	PORTB,1		; display 2
#DEFINE D3	PORTB,2		; display 3
#DEFINE D4	PORTB,4		; display 4
#DEFINE IR	PORTC,0		; IR PRIJ
#DEFINE PIEZO	PORTC,2		; PIEZO
#DEFINE OUT1_M	0x01
#DEFINE OUT2_M	0x02
#DEFINE OUT3_M	0x04
#DEFINE OUT4_M	0x08
#DEFINE OUT5_M	0x0A
#DEFINE OUT6_M	0x0C
#DEFINE TOG_BIT TOGGLE,0
#DEFINE ADDRS	0X0B        	; 0000 1011        ; 0000 1110 
#DEFINE CH1_CMD	0X01		; klic rizeni kanalu 1
#DEFINE CH2_CMD	0X02		; klic rizeni kanalu 2
#DEFINE CH3_CMD	0X03		; klic rizeni kanalu 3
#DEFINE CH4_CMD	0X04		; klic rizeni kanalu 4
#DEFINE CH5_CMD	0X05		; klic rizeni kanalu 5
#DEFINE CH6_CMD	0X06		; klic rizeni kanalu 6
;====================================
CNT1	EQU	20H
CNT2	EQU	21H
CNT3	EQU	22H
CNT4	EQU	23H
TEMP	EQU	24H
TOGGLE	EQU	25H
ADDR	EQU	26H
CMD	EQU	27H
;====================================
	ORG	000
	CALL	DEBOUNCE	
	MOVLW	B'11101000'	; presun do W 11111111
	MOVWF	TRISB		; presun do TRISB obsah W, nastaveni I/O portu
	MOVLW	B'10011001'	; presun do W 11011111
	MOVWF	TRISC		; presun do TRISC obsah W, nastaveni I/O portu
	MOVLW	B'11111111'	; presun do W 11011111
	MOVWF	TRISD		; presun do TRISB obsah W, nastaveni I/O portu
;====================================	
MAIN:	CALL	DELAY3500
	CALL	GET_RC5	
	BTFSC	STATUS,C
	GOTO	MAIN				
	MOVF	ADDR,W	
	XORLW	ADDRS
	BTFSS	STATUS,Z
	GOTO	MAIN		; ADDRESS ERROR	
	MOVF	CMD,W
	ANDLW	0X03		; MASK 2 LSB
	ADDWF	PCL,F
	GOTO	MAIN		; COMMAND 0X00 NOT USE OR COMMAND ERROR
	GOTO	DO_CH1
	GOTO	DO_CH2
	GOTO	DO_CH3
	GOTO	DO_CH4
	GOTO	DO_CH5
	GOTO	DO_CH6
;====================================	
DO_CH1: MOVLW	OUT1_M
	GOTO	CHANGE
;------------------------------------	
DO_CH2:	MOVLW	OUT2_M
	GOTO	CHANGE
;------------------------------------	
DO_CH3:	MOVLW	OUT3_M
CHANGE:	XORWF	GPIO,F	
	GOTO	MAIN
;====================================
GET_RC5:			; cteni RC5 kodu
	BTFSC	IR
	GOTO	GET_RC5		; WAIT IR LOW
	CLRF	TMR0		; clear timer 0
	NOP
	BCF	INTCON,T0IF	; CLR timer 0 flag
	NOP
	BTFSC	INTCON,T0IF     
	GOTO	ERR1		
	BTFSS	IR
	GOTO	$-6
	MOVF	TMR0,W		
	MOVWF	TEMP
	MOVLW	0XFF		; 1020 mS = 255*4
	SUBWF	TEMP,W
	BTFSC	STATUS,C
	GOTO	ERR1		; ERROR IF Time > 1020 uS
	MOVLW	0XC8		; 800 mS
	SUBWF	TEMP,W
	BTFSS	STATUS,C
	GOTO	ERR1		; ERROR IF Time < 800 uS  (C8h * TM0 Prescaler = 800 uS )
	BTFSC	IR  		; Time validate 800uS-1020uS
	GOTO	$-2		; WAIT TO LOW (Syn BIT)
	CALL	DELAY1252	; DELAY 3/4 BIT LENGHT=1252 uS
	BTFSS	IR
	BCF	TOGGLE,7	
	BTFSC	IR
	BSF	TOGGLE,7	
	BTFSS	IR
	GOTO	$+8
	BTFSC	IR
	GOTO	$-2
	GOTO	READ	
	BTFSS	IR
	GOTO	$-2
;====================================	
READ:				;cteni 5 bit adresy
	MOVLW	5
	MOVWF	TEMP		; LOOP COUNTER
	CLRF	ADDR
;------------------------------------		
READ1:	CALL	DELAY1252
	BTFSS	IR
	BCF	STATUS,C
	BTFSC	IR
	BSF	STATUS,C
	RLCF	ADDR,F		; SHIFT LEFT ADDRESS
	BTFSS	IR
	GOTO	$+8
	BTFSC	IR		; WAIT LOW
	GOTO	$-2
	GOTO	$+6
	BTFSS	IR
	GOTO	$-2
	DECFSZ	TEMP,F
	GOTO	READ1
;====================================
				; cteni 6 command bitu
	MOVLW	6
	MOVWF	TEMP		; LOOP COUNTER
	CLRF	CMD	
CMD1:	CALL	DELAY1252
	BTFSS	IR
	BCF	STATUS,C
	BTFSC	IR
	BSF	STATUS,C
	RLCF	CMD,F		; SHIFT LEFT COMMAND
	BTFSS	IR
	GOTO	$+8
	BTFSC	IR
	GOTO	$-2
	GOTO	$+6
	BTFSS	IR
	GOTO	$-2
	DECFSZ	TEMP,F
	GOTO	CMD1	
	MOVLW	0X3F		; 0011 1111
	ANDWF	CMD,F
	MOVLW	0X1F		; 0001 1111
	ANDWF	ADDR,F
	BCF	STATUS,C	; C=0 NO ERROR
	RETURN
ERR1:	BSF	STATUS,C	; C=1 HAVE ERROR	
	RETURN
;====================================
DEBOUNCE:			; ochrana zakmitu 15ms
	MOVLW	.1		; 1
	MOVWF	CNT1		; 1
DEL_1:	MOVLW	.20
	MOVWF	CNT2		; 1
DEL_2:	CLRF	CNT3		; 1
DEL_3:	DECFSZ	CNT3,F		; 1/2
	GOTO	DEL_3		; 2
	DECFSZ	CNT2,F		; 1/2
	GOTO	DEL_2		; 2
	DECFSZ	CNT1,F		; 1/2
	GOTO	DEL_1		; 2
	RETLW	0		; 2
;====================================
DELAY1252:			; prodleva 1.252ms
	MOVLW	.1		; 1
	MOVWF	CNT1		; 1
REP_1:	MOVLW	.5
	MOVWF	CNT2		; 1
REP_2:	MOVLW	.81
	MOVWF	CNT3		; 1
REP_3:	DECFSZ	CNT3,F		; 1/2
	GOTO	REP_3		; 2
	DECFSZ	CNT2,F		; 1/2
	GOTO	REP_2		; 2
	DECFSZ	CNT1,F		; 1/2
	GOTO	REP_1		; 2
	GOTO	$+2	
	GOTO	$+2	
	GOTO	$+2	
	GOTO	$+2	
	RETURN			; 2
;====================================
DELAY3500:			; prodleva 3.500ms
	MOVLW	.1		; 1
	MOVWF	CNT1		; 1
	MOVLW	.23
	MOVWF	CNT2		; 1
	MOVLW	.50
	MOVWF	CNT3
	DECFSZ	CNT3,F		; 1/2
	GOTO	$-2		; 2
	DECFSZ	CNT2,F		; 1/2
	GOTO	$-10		; 2
	DECFSZ	CNT1,F		; 1/2
	GOTO	$-18		; 2
	RETLW	0		; 2
;====================================	
	END
	
	
		
